# -*- coding: utf-8 -*-

from __future__ import absolute_import, unicode_literals
from growingio_tracker.data_parser import DataParser
from growingio_tracker.version import VERSION
from .event_v3_pb2 import CUSTOM, LOGIN_USER_ATTRIBUTES
from .event_v3_pb2 import EventV3Dto, EventV3List, ItemDto, ItemDtoList


class ProtobufParser(DataParser):

    def __init__(self):
        super(ProtobufParser, self).__init__()

    def get_bytes(self, messages, stm=None):
        return self.protobuf(messages)

    def get_headers(self):
        return {'content-type': 'application/protobuf'}

    def get_type(self, type):
        if type == "CUSTOM":
            return CUSTOM
        elif type == "LOGIN_USER_ATTRIBUTES":
            return LOGIN_USER_ATTRIBUTES
        else:
            return CUSTOM

    def protobuf(self, messages):
        message = messages[0]
        if not hasattr(message, 'event_type'):
            return self.protobuf_item(messages)
        else:
            return self.protobuf_event(messages)

    def protobuf_event(self, messages):
        events = []
        for message in messages:

            event = EventV3Dto()
            # event message
            if hasattr(message, 'event_type') and message.event_type is not None:
                event.event_type = self.get_type(message.event_type)
            if hasattr(message, 'event_time') and message.event_time is not None:
                event.timestamp = message.event_time
            if hasattr(message, 'event_name') and message.event_name is not None:
                event.event_name = message.event_name
            if hasattr(message, 'anonymous_id') and message.anonymous_id is not None:
                event.device_id = message.anonymous_id
            if hasattr(message, 'login_user_key') and message.login_user_key is not None:
                event.user_key = message.login_user_key
            if hasattr(message, 'login_user_id') and message.login_user_id is not None:
                event.user_id = message.login_user_id
            if hasattr(message, 'attributes') and message.attributes is not None:
                for k, v in message.attributes.items():
                    event.attributes[str(k)] = str(v)
            event.sdk_version = VERSION
            event.platform = 'python'
            events.append(event)

            # common
            if hasattr(message, 'data_source_id'): event.data_source_id = message.data_source_id
            if hasattr(message, 'product_id'): event.project_key = message.product_id

        event_list = EventV3List()
        event_list.values.extend(events)
        return event_list.SerializeToString()

    def protobuf_item(self, messages):
        items = []
        for message in messages:

            # item message
            item = ItemDto()
            if hasattr(message, 'key') and message.key is not None:
                item.key = message.key
            if hasattr(message, 'id') and message.id is not None:
                item.id = message.id
            if hasattr(message, 'attrs') and message.attrs is not None:
                for k, v in message.attrs.items():
                    item.attributes[str(k)] = str(v)

            # common
            if hasattr(message, 'data_source_id'): item.data_source_id = message.data_source_id
            if hasattr(message, 'product_id'): item.project_key = message.product_id

            items.append(item)

        item_list = ItemDtoList()
        item_list.values.extend(items)
        return item_list.SerializeToString()



# -*- coding: utf-8 -*-

from __future__ import absolute_import, unicode_literals
import time
import uuid

from .version import VERSION
from .default_consumer import DefaultConsumer
from .debug_consumer import DebugConsumer
from .buffered_consumer import BufferedConsumer
from .async_buffered_consumer import AsyncBufferedConsumer
from .consumer import Consumer
from .message import CustomEventMessage, ItemMessage, UserLoginMessage

__version__ = VERSION


class GrowingTracker(object):

    def __init__(self, product_id, data_source_id, server_host, consumer=None):
        self._product_id = product_id
        self._data_source_id = data_source_id
        self._server_host = server_host
        self._consumer = consumer or DefaultConsumer(product_id, data_source_id, server_host)

    @staticmethod
    def consumer(consumer):
        return GrowingTracker(consumer._product_id, consumer._data_source_id, consumer._server_host, consumer)

    @staticmethod
    def _now():
        return int(time.time() * 1000)

    @staticmethod
    def _make_insert_id():
        return uuid.uuid4().hex

    def track_custom_event(self, event_name, event_time=None, anonymous_id=None, login_user_key=None,
                           login_user_id=None,
                           attributes=None):
        message = CustomEventMessage.EventBuilder() \
            .set_product_id(self._product_id).set_data_source_id(self._data_source_id) \
            .set_event_name(event_name).set_event_time(event_time).set_anonymous_id(anonymous_id) \
            .set_login_user(login_user_key, login_user_id).set_attributes(attributes).build()
        self._consumer.send(message)

    def submit_item(self, item_key, item_id, item_attrs=None):
        message = ItemMessage.ItemBuilder().set_item_key(item_key).set_item_id(item_id) \
            .set_product_id(self._product_id).set_data_source_id(self._data_source_id) \
            .set_attributes(item_attrs).build()
        self._consumer.send(message)

    def track_user(self, login_user_key=None, login_user_id=None, anonymous_id=None, event_time=None, attributes=None):
        message = UserLoginMessage.UserBuilder() \
            .set_product_id(self._product_id).set_data_source_id(self._data_source_id) \
            .set_event_time(event_time).set_anonymous_id(anonymous_id) \
            .set_login_user(login_user_key, login_user_id).set_attributes(attributes).build()
        self._consumer.send(message)

    def track(self, message):
        message.set_product_id(self._product_id)
        message.set_data_source_id(self._data_source_id)
        self._consumer.send(message)



# -*- coding: utf-8 -*-

import time


class CustomEventMessage(object):
    def __init__(self, event_time, event_name, login_user_key, login_user_id, anonymous_id, attributes, product_id=None,
                 data_source_id=None):
        self.event_type = "CUSTOM"  # 1
        self.data_source_id = data_source_id
        self.product_id = product_id
        self.event_time = event_time
        self.event_name = event_name
        self.anonymous_id = anonymous_id
        self.login_user_key = login_user_key
        self.login_user_id = login_user_id
        self.attributes = attributes

    def set_data_source_id(self, data_source_id):
        self.data_source_id = data_source_id

    def set_product_id(self, product_id):
        self.product_id = product_id

    def __str__(self):
        message_str = '{event_type:' + self.event_type
        message_str += ',event_name:' + self.event_name
        message_str += ',product_id:' + self.product_id
        message_str += ',data_source_id:' + self.data_source_id
        message_str += ',event_time:' + str(self.event_time)
        if self.anonymous_id is not None:
            message_str += ',anonymous_id:' + self.anonymous_id
        if self.login_user_key is not None:
            message_str += ',login_user_key:' + self.login_user_key
        if self.login_user_id is not None:
            message_str += ',login_user_id:' + self.login_user_id
        if self.attributes is not None:
            message_str += ',attributes:' + str(self.attributes)
        message_str += '}'
        return message_str

    class EventBuilder(object):
        def __init__(self):
            self.data_source_id = None
            self.product_id = None
            self.event_time = int(time.time() * 1000)
            self.event_name = None
            self.anonymous_id = None
            self.login_user_key = None
            self.login_user_id = None
            self.attributes = None
            pass

        def set_data_source_id(self, data_source_id):
            self.data_source_id = data_source_id
            return self

        def set_product_id(self, product_id):
            self.product_id = product_id
            return self

        def set_event_time(self, timestamp):
            if timestamp is not None:
                self.event_time = timestamp
            return self

        def set_event_name(self, name):
            self.event_name = name
            return self

        def set_anonymous_id(self, device_id):
            self.anonymous_id = device_id
            return self

        def set_login_user(self, user_key=None, user_id=None):
            self.login_user_id = user_id
            self.login_user_key = user_key
            return self

        def set_login_user_key(self, user_key):
            self.login_user_key = user_key
            return self

        def set_login_user_id(self, user_id):
            self.login_user_id = user_id
            return self

        def add_attribute(self, key, value):
            if self.attributes is None:
                self.attributes = {}
            self.attributes[key] = value
            return self

        def set_attributes(self, map):
            self.attributes = map
            return self

        def build(self):
            return CustomEventMessage(self.event_time, self.event_name,
                                      self.login_user_key, self.login_user_id,
                                      self.anonymous_id, self.attributes,
                                      self.data_source_id, self.product_id)


class UserLoginMessage(object):
    def __init__(self, login_user_key, login_user_id, event_time, anonymous_id, attributes, product_id=None,
                 data_source_id=None):
        self.event_type = "LOGIN_USER_ATTRIBUTES"  # 3
        self.data_source_id = data_source_id
        self.product_id = product_id
        self.login_user_key = login_user_key
        self.login_user_id = login_user_id
        self.event_time = event_time
        self.anonymous_id = anonymous_id
        self.attributes = attributes

    def set_data_source_id(self, data_source_id):
        self.data_source_id = data_source_id

    def set_product_id(self, product_id):
        self.product_id = product_id

    def __str__(self):
        message_str = '{event_type:' + self.event_type
        message_str += ',product_id:' + self.product_id
        message_str += ',data_source_id:' + self.data_source_id
        message_str += ',event_time:' + str(self.event_time)
        if self.anonymous_id is not None:
            message_str += ',anonymous_id:' + self.anonymous_id
        if self.login_user_key is not None:
            message_str += ',login_user_key:' + self.login_user_key
        if self.login_user_id is not None:
            message_str += ',login_user_id:' + self.login_user_id
        if self.attributes is not None:
            message_str += ',attributes:' + str(self.attributes)
        message_str += '}'
        return message_str

    class UserBuilder(object):
        def __init__(self):
            self.data_source_id = None
            self.product_id = None
            self.event_time = int(time.time() * 1000)
            self.anonymous_id = None
            self.login_user_key = None
            self.login_user_id = None
            self.attributes = None

        def set_data_source_id(self, data_source_id):
            self.data_source_id = data_source_id
            return self

        def set_product_id(self, product_id):
            self.product_id = product_id
            return self

        def set_event_time(self, timestamp):
            self.event_time = timestamp
            return self

        def set_anonymous_id(self, device_id):
            self.anonymous_id = device_id
            return self

        def set_login_user(self, user_key=None, user_id=None):
            self.login_user_id = user_id
            self.login_user_key = user_key
            return self

        def set_login_user_key(self, user_key):
            self.login_user_key = user_key
            return self

        def set_login_user_id(self, user_id):
            self.login_user_id = user_id
            return self

        def add_attribute(self, key, value):
            if self.attributes is None:
                self.attributes = {}
            self.attributes[key] = value
            return self

        def set_attributes(self, map):
            self.attributes = map
            return self

        def build(self):
            return UserLoginMessage(self.login_user_key, self.login_user_id,
                                    self.event_time, self.anonymous_id,
                                    self.attributes, self.data_source_id, self.product_id)


class ItemMessage(object):
    def __init__(self, item_key, item_id, attrs=None, product_id=None, data_source_id=None, ):
        self.data_source_id = data_source_id
        self.product_id = product_id
        self.key = item_key
        self.id = item_id
        self.attrs = attrs

    def set_data_source_id(self, data_source_id):
        self.data_source_id = data_source_id

    def set_product_id(self, product_id):
        self.product_id = product_id

    def __str__(self):
        message_str = '{item_key:' + str(self.key)
        message_str += ',item_id:' + str(self.id)
        message_str += ',product_id:' + self.product_id
        message_str += ',data_source_id:' + self.data_source_id
        if self.attrs is not None:
            message_str += ',attributes:' + str(self.attrs)
        message_str += '}'
        return message_str

    class ItemBuilder(object):
        def __init__(self):
            self.data_source_id = None
            self.product_id = None
            self.id = None
            self.key = None
            self.attrs = None

        def set_data_source_id(self, data_source_id):
            self.data_source_id = data_source_id
            return self

        def set_product_id(self, product_id):
            self.product_id = product_id
            return self

        def set_item_id(self, item_id):
            self.id = item_id
            return self

        def set_item_key(self, item_key):
            self.key = item_key
            return self

        def add_attribute(self, key, value):
            if self.attrs is None:
                self.attrs = {}
            self.attrs[key] = value
            return self

        def set_attributes(self, map):
            self.attrs = map
            return self

        def build(self):
            return ItemMessage(self.key, self.id, self.attrs, self.data_source_id, self.product_id)



# -*- coding: utf-8 -*-

from .data_parser import JsonParser
from .consumer import Consumer


class DefaultConsumer(Consumer):
    def __init__(self, product_id, data_source_id, server_host, data_parser=None,
                 retry_limit=3, request_timeout=5, retry_backoff_factor=0.25, verify_cert=True):
        super(DefaultConsumer, self).__init__(product_id, data_source_id, server_host,
                                              retry_limit, request_timeout, retry_backoff_factor, verify_cert)
        self._data_parse = data_parser or JsonParser()

    def send(self, message):
        if hasattr(message, 'event_type'):
            request_url = self.endpoints['collect']
        else:
            request_url = self.endpoints['item']

        messages = [message]
        self.post_data(request_url, messages)

    def post_data(self, request_url, messages):
        stm = self._data_parse.get_timestamp()
        data = self._data_parse.get_bytes(messages, stm)

        self.send_data(
            request_url,
            data,
            params=self._data_parse.get_params(stm=stm),
            headers=self._data_parse.get_headers()
        )
